
/**
 * View
 * @class View
 * @param {Object} options Default options for view
 * @since 0.0.1
 * @author Jerome Vial
 *
 * @type {Class}
 */
define([
	'ViewCore/ctrl',
	'UI/Container/Container',
	'UI/Layout/Layout',
	'Core/Module/Toolbar/Toolbar',
	'Core/Module/Binding/Binding',
	'ViewCore/Container',
	//'ViewCore/ctrl',
	'ViewCore/Dragging',
	'ViewCore/Limit',
	'ViewCore/Loader',
	'ViewCore/Scroll',
	'ViewCore/Layout',
	//'ViewCore/Scrolling',
	'ViewCore/Zoom'
], function(
	viewCtrl,
	UIContainer,
	Layout,
	Toolbar,
	Binding,
	Container,
	//ctrl,
	Dragging,
	Limit,
	Loader,
	Scroll,
	LayoutView,
	//Scrolling,
	Zoom
) {

	var exports = Minimal.View = new Class({

		Implements: [
			Events,
			Options,
			Binding,
			Toolbar,

			Container,
			//ctrl,
			Dragging,
			Limit,
			Loader,
			Scroll,
			LayoutView,
			//Scrolling,
			Zoom
		],

		options: {
			lib: 'minimal',
			base: 'view',
			clss: 'view',

			content: true,

			friction: 5,
			tag: 'div',
			node: {
				name: 'container'
			},
			axis: {
				x: true,
				y: true
			},

			range: [10, 52],

			fx: {
				transition: Fx.Transitions.Quart.easeOut,
				duration: 240,
				wait: false
			},
			scrollbar: null,
			scroll: 'scrollbar',
			dragging: null,
			scroller: null,
			scroller_opts: {
				area: 160,
				velocity: 0.5
			},
			autoScrollOptions: {
				wait: false,
				duration: 200,
				offset: {'x': 0, 'y': -50},
				transition: Fx.Transitions.Quad.easeInOut
			},
			controller: {
				_list: []
			}
		},


		/**
		 * Initialize
		 *
		 * @method initialize
		 * @param {options}
		 * @return {this}
		 * @private
		 */
		initialize: function(options) {
			//_log.debug('Minimal.View.initialize', options);
			options = options || {};
			this.sandbox = options.sandbox;
			delete options.sandbox;

			this.setOptions(options);
			this.opts = this.options;

			this.container = options.container;

			this._init();

			return this;
		},

		/**
		 * Initialize
		 *
		 * @method _init
		 * @param {Object}
		 * @return {void}
		 * @private
		 */
		_init: function(opts) {
			//_log.debug('init');
			opts = this.opts;

			this.index = 0;
			this.views = [];
			//this.layout = ui.node[opts.name];

			this._initContainer();

			if (opts.toolbar)
				this._initToolbar(opts.toolbar);

			if (opts.menus)
				this._initMenus();

			this._initEvents();

			if (opts.scroller)
				this._initScroller();

			this._initView();
			//this.binding = this._initBinding();

			viewCtrl.register(this);

			this.fireEvent('initReady');
		},

		/**
		 * Check Local Data
		 *
		 * @method _checkLocalData
		 * @param {node_id, type} node_id, type
		 * @return {void}
		 * @private
		 */
		_initView: function(){
			var self = this,
				opts = this.options;

			this.isOpen = true;
			this.visible = true;

			/*if (opts.foot)
				this.container._initFoot();

			this.foot = this.container.foot;*/
		},

		/**
		 * Initialize
		 *
		 * @method _initOptions
		 * @param {Object}
		 * @return {this}
		 * @private
		 */
		_init__Options: function(opts){
			//this.name = this.options.name;
			this.main = opts.main || opts.name;

			ui.node = ui.node || {};
			ui.node[this.main] = ui.node[this.main] || {};

			this.layout = opts.layout || {};
			this.layout[this.main] = this.layout[this.main] || {};

			this.dragHandlers = opts.dragHandlers || [];

			return this;
		},

		/**
		 * Initialize Content
		 *
		 * @method _initContent
		 * @return {void}
		 * @private
		 */
		_initContent: function(){
			var self = this,
				opts = this.options;

			/**
			 * Fired before element is created.
			 * @param {Ext.Component} this
			 * @param {String} value
			 */
			this.fireEvent('create');

			// should call UI.Component
			var element = new Element(opts.tag, {
				'class': this.klass,
				styles: opts.styles,
				events: opts.events,
				id: opts.id,
				name: opts.name,
				html: opts.html
			});

			element.addClass('container-body');
			//element.addClass('view-' + opts.name);
			// element.addEvents({
			// 	mouseup: function(e) {
			// 		//_log.debug('----focus', self.options.name);
			// 		//self.focus(e);
			// 	}
			// });

			this.element = element;
			this.content = element;

			element.store('_instance', this);

			/**
			 * Fired when element is clicked on.
			 * @param {Ext.Component} this
			 * @param {String} value
			 */
			this.fireEvent('created');
			// inject the component if the container is given

			if ( opts.container && opts.container != 'window')
				this.element.inject(this.container);

			if ( opts.resizable )
				this._initResizer();

			//this._initState();
			this._initClass();
		},

		/**
		 * [focus description]
		 * @return {void}
		 */
		focus: function(){
			//_log.debug('focus');
			viewCtrl.focus(this);
			this.fireEvent('focus');
			this.fireEvent('render');
		},

		/**
		 * Initialize The User Interface
		 *
		 * @method _initLayout
		 * @return {void}
		 * @private
		 */
		_initLayout: function(layout) {
			_log.debug('--!!!', layout);
			//this.parent();
			var opts = this.options;

			this.container = new UIContainer({
				container: opts.container,
				name: opts.name,
				node: opts.layout || opts.node
			});

			this.layout = new Layout({
				container: this.container,
				layout: opts.layout
			});

			this.container = this.layout.container;
		},

		/**
		 * This should in
		 * @return {void}
		 * @private
		 */
		_initConnector: function() {
			var self = this,
				opts = this.opts;

			this.conn = opts.conn;
			//this.data.get();

			this.conn.addEvents({
				ready: function(feed) {

				}
			});
		},

		/**
		 * [_initConnector description]
		 * @return {void}
		 * @private
		 */
		_connect: function(conn) {
			var opts = this.opts,
				self = this;

			//_log.debug('connn', conn);

			if (!conn) return;

			//_log.debug('conn exists');

			this.data = conn;
			this.data.get();

			this.data.addEvents({
				ready: function(data) {
					self.fireEvent('dataReady', data);
				}
			});

		},

		/**
		 * [_initModules description]
		 * @return {void}
		 * @private
		 */
		_initModules: function(modules, callback) {
			//_log.debug('initModule');

			require(modules, function() {
				callback();
			});
		},

		/**
		 * Initialize Events
		 *
		 * @method _initEvents
		 * @return {void}
		 * @private
		 */
		_initEvents : function(){
			//this.parent();

			var self = this,
				opts = this.options;

			this.fx = new Fx.Scroll(this.element, this.options.fx);

			if (this._initLoader)
				this._initLoader();

			/*
			if (opts.scrollbar)
				this._initScrollbar();
			*/

			if (opts.dragging)
				this._initDragging();

			if (!opts.container) return;

			//if (opts.container.component != 'window')
			/*this.container.addEvent('resize', function() {
				_log.debug('resize from view.initEvents');
				self.fireEvent('resize');
			});*/
		},

		/**
		 * Initialize Class
		 *
		 * @method _initClass
		 * @return {void}
		 * @private
		 */
		_initClass: function(){
			var opts = this.options;

			this.element.addClass(opts.lib + '-' + opts.base);
			this.element.addClass(opts.base + '-' + opts.clss);

			if (opts.klss)
				this.element.addClass(opts.klss);
		},

		/**
		 * Destroy Element
		 *
		 * @method remove
		 * @param {element}
		 */
		remove: function(element){
			if (element.destroy) {
				element.destroy();
			}
		},

		/**
		 * Set Title
		 *
		 * @method setTitle
		 * @param {text}
		 * @return {this}
		 */
		setTitle: function(text) {
		    //_log.debug('setTitle', text, this.control.title);
		    var self = this;

		    (function() {
		        if (self.control.title) {
		            self.control.title.set(text);
		        }
		    }).delay(200);

		    // if (this.container.head)
		    // 	this.container.setTitle(text);

		    return this;
		},

		/**
		 * Set Title
		 *
		 * @method setTitle
		 * @param {text}
		 * @return {String}
		 */
		getTitle: function(text) {
			//_log.debug('getTitle', this.container.getTitle());

			return this.container.getTitle();
		},

		/**
		 * Set Status
		 *
		 * @method setStatus
		 * @param {text}
		 * @private
		 */
		setStatus: function(text) {
			//_log.debug(text);
			if (this.control && this.control.status)
				this.control.status.element.set('html', text);
			//this.container.fireEvent('resize');
		},

		/**
		 * Return the isVisible status
		 *
		 * @return {Boolean}
		 */
		isVisible: function() {
			return this.visible;
		},

		/**
		 * Clear the content of the view
		 * @return {[type]} [description]
		 */
		clear: function() {
			if (this.content && this.content.empty) {
				this.content.empty();
			}
		},

		/**
		 * Close
		 *
		 * @method close
		 * @return {this}
		 * @private
		 */
		close: function() {
			this.container.close();

			return this;
		},

		/**
		 * Initialize Status
		 *
		 * @method _initStatus
		 * @return {void}
		 * @private
		 */
		_initStatus: function() {

		},

		/**
		 * Initialize Data
		 *
		 * @method _initData
		 * @return {void}
		 * @private
		 */
		_initData: function() {


		},

		/**
		 * [hide description]
		 * @return {[type]} [description]
		 */
		hide: function() {
			//_log.debug('hide', this.container);
			this.container.hide();
			this.visible = false;
		},

		/**
		 * [show description]
		 * @return {[type]} [description]
		 */
		show: function() {
			//_log.debug('close', this.container);
			this.container.setStyle('display', null);
			this.visible = true;
		}

	});

	return exports;

});
